pub struct TerminalCache { /* private fields */ }Expand description
Terminal cache for storing expensive computations
Implementations
Sourceimpl TerminalCache
impl TerminalCache
Sourcepub fn get_sys_info(&self) -> Option<SysInfo>
pub fn get_sys_info(&self) -> Option<SysInfo>
Get cached system info if not expired (TTL: 5 seconds)
Sourcepub fn set_sys_info(&self, info: SysInfo)
pub fn set_sys_info(&self, info: SysInfo)
Cache system info
Sourcepub fn get_mining_info(&self) -> Option<MiningInfo>
pub fn get_mining_info(&self) -> Option<MiningInfo>
Get cached mining info if not expired (TTL: 10 seconds)
Sourcepub fn set_mining_info(&self, info: MiningInfo)
pub fn set_mining_info(&self, info: MiningInfo)
Cache mining info
Sourcepub fn get_tx_pool_info(&self) -> Option<TerminalPoolInfo>
pub fn get_tx_pool_info(&self) -> Option<TerminalPoolInfo>
Get cached transaction pool info if not expired (TTL: 2 seconds)
Sourcepub fn set_tx_pool_info(&self, info: TerminalPoolInfo)
pub fn set_tx_pool_info(&self, info: TerminalPoolInfo)
Cache transaction pool info
Sourcepub fn get_cells_info(&self) -> Option<CellsInfo>
pub fn get_cells_info(&self) -> Option<CellsInfo>
Get cached cells info if not expired (TTL: 30 seconds)
Sourcepub fn set_cells_info(&self, info: CellsInfo)
pub fn set_cells_info(&self, info: CellsInfo)
Cache cells info
Sourcepub fn get_network_info(&self) -> Option<NetworkInfo>
pub fn get_network_info(&self) -> Option<NetworkInfo>
Get cached network info if not expired (TTL: 10 seconds)
Sourcepub fn set_network_info(&self, info: NetworkInfo)
pub fn set_network_info(&self, info: NetworkInfo)
Cache network info
Sourcepub fn clear_specific(&self, refresh: RefreshKind)
pub fn clear_specific(&self, refresh: RefreshKind)
Clear specific cache entry types based on refresh flags